
# =====================================================================================================================
# **NOTICE**
# THIS IS A GENERATED FILE USING THE SOURCE FILE:
#  crab\utilities\java\seeq-utilities\src\main\java\com\seeq\utilities\SeeqNames.java
# =====================================================================================================================

class SeeqNames:
    class Injection:
        everyone_user_group = 'EveryoneUserGroup'
        system_state = 'SystemState'
        system_user = 'SystemUser'
    
    class Connectors:
        connections_json_key = 'Connections'
        datasource_managed_json_key = 'DatasourceManaged'
        class Connections:
            name_json_key = 'Name'
            datasource_id_json_key = 'Id'
            enabled_json_key = 'Enabled'
            indexing_json_key = 'Indexing'
            frequency_json_key = 'Frequency'
            next_indexing_at_json_key = 'Next'
            on_startup_and_config_change_json_key = 'OnStartupAndConfigChange'
            transforms_json_key = 'Transforms'
            max_concurrent_requests_json_key = 'MaxConcurrentRequests'
            max_results_per_request_json_key = 'MaxResultsPerRequest'
            class Status:
                connecting = 'CONNECTING'
                connected = 'CONNECTED'
                disconnected = 'DISCONNECTED'
                disabled = 'DISABLED'
            
            class SyncMode:
                none = 'NONE'
                incremental = 'INCREMENTAL'
                full = 'FULL'
            
            class SyncResult:
                in_progress = 'IN_PROGRESS'
                success = 'SUCCESS'
                failed = 'FAILED'
                interrupted = 'INTERRUPTED'
            
        
    
    class Agents:
        connectors_json_key = 'Connectors'
        class Connectors:
            name_json_key = 'Name'
            enabled_json_key = 'Enabled'
        
    
    class TreeType:
        seeq_workbench = 'Seeq Workbench'
    
    class OData:
        export_entity_set_suffix = '_DataSet'
    
    class CleanUpJobsGroup:
        annotation_images = 'AnnotationOrphanedImageCleanerJob'
        worksheet_maintenance = 'WorksheetMaintenanceJob'
    
    class JobGroup:
        screenshot = 'ScreenshotCapture'
        date_range = 'DateRangeJobs'
        report = 'ReportJobs'
        notebook = 'NotebookJobs'
        clean_up = 'CleanUpJobs'
        report_template = 'ReportTemplateJobs'
    
    class Logging:
        migrations_complete_file = 'migrations_complete_'
        class RemoteAgents:
            m_d_c_filename_key = 'agentlogfilename'
        
        class Appserver:
            m_d_c_filename_key = 'logfilename'
            default_file = 'appserver'
            startup_file = 'appserver-startup'
            upgrade_file = 'appserver-upgrade'
            async_upgrade_file = 'appserver-async-upgrade'
            licensing_file = 'appserver-licensing'
            requests = 'appserver-requests'
            request_monitors = 'appserver-requestmonitors'
            units = 'appserver-units'
            users = 'appserver-users'
            class IntervalAnalysis:
                fetch_critic = 'appserver-intervalanalysis-fetchcritic'
            
        
    
    class RemoteDatasourceClasses:
        l_d_a_p = 'LDAP'
        o_auth2 = 'OAuth 2.0'
        windows_auth = 'Windows Auth'
        a_d_x = 'ADX'
        big_query = 'BigQuery'
        chaos_monkey = 'Chaos Monkey'
        cognite = 'Cognite'
        cyg_net = 'CygNet'
        example_data = 'Time Series CSV Files'
        add_on_calc_python = 'Add-on Calculation Python'
        add_on_calc_excel = 'Add-on Calculation Excel VBA'
        file_folders = 'FileFolders'
        g_e_proficy = 'GE Proficy'
        influx_d_b = 'InfluxDB'
        influx_d_b2 = 'InfluxDB 2'
        i_p21 = 'AspenTech IP21'
        met_net = 'MetNet'
        n_o_a_a_weather = 'NOAA Weather Service'
        o_p_c_h_d_a = 'OPC-HDA'
        o_p_c_u_a = 'OPC-UA'
        o_s_isoft_a_f = 'OSIsoft AF'
        o_s_isoft_a_f_event_frames = 'OSIsoft AF Event Frames'
        o_s_isoft_p_i = 'OSIsoft PI'
        s_q_l = 'SQL'
        stress_test = 'Stress Test'
        time_stream = 'TimeStream'
        tree_file = 'Tree File'
        w_i_t_s_m_l = 'WITSML'
        wonderware = 'Wonderware'
        x_h_q = 'XHQ'
    
    class LocalDatasources:
        class SeeqMetadataItemStorage:
            datasource_class = 'Seeq Metadata'
            datasource_id = 'Seeq Metadata'
            datasource_name = 'Seeq Metadata'
        
        class SeeqCalculationItemStorage:
            datasource_class = 'Seeq Calculations'
            datasource_id = 'Seeq Calculations'
            datasource_name = 'Seeq Calculations'
        
        class Monitors:
            datasource_class = 'Monitors'
            datasource_id = 'monitors'
            datasource_name = 'Seeq Monitors'
        
        class Authentication:
            datasource_class = 'Auth'
            datasource_id = 'Seeq'
            datasource_name = 'Seeq'
        
        class FileSignalStorage:
            datasource_class = 'Seeq - Signal Datasource'
            datasource_id = 'Signal Datasource'
            datasource_name = 'Seeq Signal Datasource'
        
        class FileSignalCache:
            datasource_class = 'Seeq - FileSignal'
            datasource_id = 'default'
            datasource_name = 'Seeq Cache'
        
        class PostgresDatums:
            datasource_class = 'Postgres Datums'
            datasource_id = 'default'
            datasource_name = 'Seeq Conditions Database'
        
        class Datafile:
            datasource_class = 'Imported CSV Files'
            datasource_id = 'Imported CSV Files'
            datasource_name = 'Imported CSV Files'
        
    
    class SwaggerApiNames:
        agents = 'Agents'
        add_ons = 'AddOns'
        audit = 'Audit'
        auth = 'Auth'
        assets = 'Assets'
        folders = 'Folders'
        networks = 'Networks'
        items = 'Items'
        reports = 'Reports'
        jobs = 'Jobs'
        logs = 'Logs'
        trees = 'Trees'
        sample_series = 'SampleSeries'
        signals = 'Signals'
        capsules = 'Capsules'
        conditions = 'Conditions'
        metrics = 'Metrics'
        formulas = 'Formulas'
        functions = 'Functions'
        requests = 'Requests'
        tables = 'Tables'
        workbooks = 'Workbooks'
        worksheets = 'Worksheets'
        export = 'Export'
        annotations = 'Annotations'
        datasources = 'Datasources'
        datafiles = 'Datafiles'
        users = 'Users'
        user_groups = 'UserGroups'
        access_keys = 'AccessKeys'
        usage = 'Usage'
        scalars = 'Scalars'
        subscriptions = 'Subscriptions'
        system = 'System'
        import_ = 'Import'
        monitors = 'Monitors'
        projects = 'Projects'
        content = 'Content'
        report_templates = 'ReportTemplates'
        plugins = 'Plugins'
        displays = 'Displays'
        display_templates = 'DisplayTemplates'
    
    class Channels:
        agents_status = 'agents-status'
        datasources_status = 'datasources-status'
        broadcast = 'broadcast'
        live_screenshot = 'live-screenshot'
        async_response = 'async-response'
        requests_progress = 'requests-progress'
        workbook_channel = 'workbook'
        workstep_channel = 'workstep'
        report_update_channel = 'report-update'
    
    class API:
        ace = '/ace'
        add_ons = '/add-ons'
        agents = '/agents'
        audit = '/audit'
        auth = '/auth'
        assets = '/assets'
        folders = '/folders'
        networks = '/networks'
        items = '/items'
        reports = '/reports'
        jobs = '/jobs'
        logs = '/logs'
        batch = '/batch'
        trees = '/trees'
        asset_trees = '/trees/assets'
        signals = '/signals'
        samples = '/samples'
        plot_samples = '/plot-samples'
        capsules = '/capsules'
        conditions = '/conditions'
        metrics = '/metrics'
        thresholds = '/thresholds'
        formulas = '/formulas'
        docs = '/docs'
        functions = '/functions'
        packages = '/packages'
        scalars = '/scalars'
        properties = '/properties'
        requests = '/requests'
        tables = '/tables'
        workbooks = '/workbooks'
        worksheets = '/worksheets'
        worksteps = '/worksteps'
        export = '/export'
        annotations = '/annotations'
        datasources = '/datasources'
        datafiles = '/datafiles'
        users = '/users'
        user_groups = '/usergroups'
        access_keys = '/accesskeys'
        o_data = '/odata.svc'
        system = '/system'
        import_ = '/import'
        status = '/status'
        screenshots = '/screenshots'
        subscriptions = '/subscriptions'
        monitors = '/system/monitors'
        configuration = '/system/configuration'
        installer = '/system/installer'
        projects = '/projects'
        content = '/content'
        report_templates = '/report-templates'
        date_ranges = '/date-ranges'
        asset_selections = '/asset-selections'
        plugins = '/plugins'
        displays = '/displays'
        display_templates = '/display-templates'
        usage = '/usage'
        class ErrorMessages:
            attempted_to_set_scope_on_a_globally_scoped_item = 'Attempted to set scope on a globally scoped item'
        
        class AddOnToolLinkType:
            window = 'window'
            tab = 'tab'
            none = 'none'
        
        class PropertyIds:
            request_information = 'com.seeq.requestinformation'
        
        class Cookies:
            auth = 'sq-auth'
            csrf = 'sq-csrf'
        
        class QueryParams:
            client_interactive_session_id = 'interactiveId'
            request_id = 'requestId'
        
        class Headers:
            agent_id = 'x-sq-agent-id'
            async_ = 'x-sq-async'
            auth = 'x-sq-auth'
            csrf = 'x-sq-csrf'
            forwarded_for = 'x-forwarded-for'
            interactive_session_id = 'Session-Id'
            request_id = 'x-sq-request-id'
            request_origin = 'x-sq-origin'
            request_origin_u_r_l = 'x-sq-origin-url'
            request_origin_label = 'x-sq-origin-label'
            server = 'Server'
            server_meters = 'Server-Meters'
            server_timing = 'Server-Timing'
            expect = 'Expect'
            date_range_start = 'x-sq-daterange-start'
            date_range_end = 'x-sq-daterange-end'
            class Timings:
                datasource = 'Datasource'
                cache = 'Cache'
                processing = 'Processing'
                g_c = 'GC'
                request_queue = 'Request Queue'
                calc_engine_queue = 'Calc Engine Queue'
                seeq_database = 'Seeq Database'
                total = 'Total'
            
            class Meters:
                datasource_samples_read = 'Datasource Samples Read'
                datasource_capsules_read = 'Datasource Capsules Read'
                cache_samples_read = 'Cache Samples Read'
                cache_capsules_read = 'Cache Capsules Read'
                cache_in_memory_samples_read = 'Cache In-Memory Samples Read'
                cache_in_memory_capsules_read = 'Cache In-Memory Capsules Read'
                database_items_read = 'Database Items Read'
                database_relationships_read = 'Database Relationships Read'
            
        
    
    class UnitTest:
        timestamp1 = '1990-03-07T09:58:20.888888Z'
        timestamp2 = '1990-04-08T10:59:21.999999Z'
        non_existent_guid = '1CB64296-FC3E-4221-9A7F-947A0AB99807'
    
    class Edges:
        owns = 'owns'
        created = 'created'
        installed = 'installed'
        contains = 'contains'
        folder_contains = 'folder contains'
        member_of = 'member of'
        interested_in = 'interested in'
        reports_on = 'reports on'
        replies_to = 'replies to'
        recently_accessed = 'recently accessed'
        has_worksheet = 'has worksheet'
        has_workstep = 'has workstep'
        current_workstep = 'current workstep'
        has_parameter = 'has parameter'
        uses_function = 'uses function'
        associated_to = 'associated to'
        # DEPRECATED, Deprecated. See SeeqNames.java for more info
        access_for = 'access for'
        # DEPRECATED, Deprecated. See SeeqNames.java for more info
        has_ace = 'has access control entry'
        has_pinned = 'has pinned'
        measuring = 'measuring'
        has_aggregation_signal = 'has aggregation signal'
        has_aggregation_condition = 'has aggregation condition'
        has_aggregation_formula_function = 'has aggregation formula function'
        bounded_by = 'bounded by'
        has_threshold = 'has threshold'
        has_condition_threshold = 'has condition threshold'
        home_folder = 'home folder'
        package_contains_u_d_f = 'package contains user defined function'
        package_contains_doc = 'package contains formula doc'
        documented_by = 'documented by'
        agent_contains_connector = 'agent contains connector'
        connection_created_by_connector = 'connection created by connector'
        renders = 'renders'
        runs_notebook = 'runs notebook'
        source_worksheet = 'source worksheet'
        source_workstep = 'source workstep'
        has_asset = 'has asset'
        has_asset_selection = 'has asset selection'
        has_asset_selections = 'has asset selections'
        has_condition = 'has condition'
        has_content = 'has content'
        has_date_range = 'has date range'
        has_date_ranges = 'has date ranges'
        has_scheduled_notebook = 'has scheduled notebook'
        has_template = 'has template'
    
    class Types:
        asset = 'Asset'
        asset_selection = 'AssetSelection'
        access_control_entry = 'AccessControlEntry'
        stored_signal = 'StoredSignal'
        datasource = 'Datasource'
        # DEPRECATED, Deprecated. See SeeqNames.java for more info
        table_datasource = 'TableDatasource'
        datafile = 'Datafile'
        calculated_signal = 'CalculatedSignal'
        signal = 'Signal'
        tree_definition = 'TreeDefinition'
        identity = 'Identity'
        user = 'User'
        user_group = 'UserGroup'
        access_key = 'AccessKey'
        system_state = 'SystemState'
        export = 'Export'
        project_link = 'ProjectLink'
        add_on_tool = 'AddOnTool'
        base_annotation = 'BaseAnnotation'
        report = 'Report'
        report_template = 'ReportTemplate'
        journal = 'Journal'
        reply = 'Reply'
        condition = 'Condition'
        capsule = 'Capsule'
        stored_condition = 'StoredCondition'
        calculated_condition = 'CalculatedCondition'
        calculated_scalar = 'CalculatedScalar'
        formula_function = 'FormulaFunction'
        aggregating_formula_function = 'AggregatingFormulaFunction'
        chart = 'Chart'
        user_defined_formula_function = 'UserDefinedFormulaFunction'
        formula_package = 'FormulaPackage'
        formula_doc = 'FormulaDoc'
        formula_package_doc = 'FormulaPackageDoc'
        threshold_metric = 'ThresholdMetric'
        display = 'Display'
        display_template = 'DisplayTemplate'
        folder = 'Folder'
        project = 'Project'
        content = 'Content'
        image_content = 'ImageContent'
        react_json_content = 'ReactJsonContent'
        date_range = 'DateRange'
        scheduled_notebook = 'ScheduledNotebook'
        add_on = 'AddOn'
        plugin = 'Plugin'
        connection = 'Connection'
        connector = 'Connector'
        agent = 'Agent'
        workbook = 'Workbook'
        analysis = 'Analysis'
        topic = 'Topic'
        worksheet = 'Worksheet'
        workstep = 'Workstep'
        item = 'Item'
        item_with_owner = 'ItemWithOwner'
        type_definition = 'TypeDefinition'
    
    class TopicDocumentAttributes:
        data_seeq_content = 'data-seeq-content'
        data_seeq_content_pending = 'data-seeq-content-pending'
        data_seeq_date_range_id = 'data-date-range-id'
        data_seeq_date_range_format = 'data-date-range-format'
        data_seeq_date_range_content = 'data-date-range-content'
        data_seeq_content_percent_width = 'data-seeq-content-percent-width'
        data_seeq_content_no_margin = 'data-seeq-content-no-margin'
        data_seeq_content_border = 'data-seeq-content-border'
        data_seeq_content_height = 'data-seeq-content-height'
        data_seeq_content_width = 'data-seeq-content-width'
        data_seeq_content_property_overrides = 'data-seeq-content-property-overrides'
        data_seeq_asset_selection_id = 'data-asset-selection-id'
        data_seeq_asset_selection_depth_level = 'data-asset-selection-depth-level'
        data_seeq_iframe_content_url = 'data-iframe-content-url'
        data_seeq_iframe_content_height = 'data-iframe-content-height'
        data_seeq_iframe_content_width = 'data-iframe-content-width'
    
    class CapsuleProperties:
        start = 'Start'
        end = 'End'
        duration = 'Duration'
        middle = 'Middle'
        count = 'Count'
        value = 'Value'
        unit = 'Unit'
        distance = 'Distance'
        similarity = 'Similarity'
        standard_deviation = 'Standard Deviation'
        sample_period = 'Sample Period'
        reference_capsule = 'Reference Capsule'
        condition_id = 'Condition ID'
        capsule_id = 'Capsule ID'
        capsule_id_safe = 'Capsule SortKey'
        original_uncertainty = 'Original Uncertainty'
    
    class Properties:
        guid = 'GUID'
        name = 'Name'
        id = 'ID'
        description = 'Description'
        unsearchable = 'Unsearchable'
        archived = 'Archived'
        archival_id = 'Archival ID'
        archived_at = 'Archived At'
        archived_reason = 'Archived Reason'
        sync_token = 'Sync Token'
        number_format = 'Number Format'
        source_number_format = 'Source Number Format'
        string_format = 'String Format'
        source_string_format = 'Source String Format'
        storage_location = 'Storage Location'
        creator_id = 'Creator ID'
        creator_first_name = 'Creator First Name'
        creator_last_name = 'Creator Last Name'
        historical_data_version = 'Historical Data Version'
        manual_cdc = 'Manual CDC'
        scoped_to = 'Scoped To'
        import_item_type = 'Import Item Type'
        filename = 'Filename'
        name_row = 'Name Row'
        name_prefix = 'Name Prefix'
        name_suffix = 'Name Suffix'
        condition_name = 'Condition Name'
        first_data_row = 'First Data Row'
        description_row = 'Description Row'
        interpolation_method_row = 'Interpolation Method Row'
        maximum_interpolation_row = 'Maximum Interpolation Row'
        value_uom_row = 'Value Unit of Measure Row'
        field_delimiter = 'Field Delimiter'
        day_first_default = 'Day First Default'
        key_column_name = 'Key Column Name'
        key_column_index = 'Key Column Index'
        end_column_name = 'End Column Name'
        end_column_index = 'End Column Index'
        key_format = 'Key Format'
        time_zone = 'Time Zone'
        value_column_names = 'Value Column Names'
        value_column_indices = 'Value Column Indices'
        append = 'Append'
        validation_mode = 'Validation Mode'
        lenient_daylight_savings = 'Lenient Daylight Savings'
        datasource_class = 'Datasource Class'
        datasource_name = 'Datasource Name'
        datasource_id = 'Datasource ID'
        datasource_host_id = 'Datasource Host ID'
        data_id = 'Data ID'
        stored_in_seeq = 'Stored In Seeq'
        connection_request_timeout = 'Connection Request Timeout'
        max_requests = 'Max Requests'
        windows_auth_mode = 'Windows Auth Mode'
        ldap_auth_mode = 'LDAP Auth Mode'
        username_password_compatible = 'Username/Password Compatible'
        username_human_readable = 'Username Human Readable'
        expect_duplicates_during_indexing = 'Expect Duplicates During Indexing'
        referenced_datasource_id = 'Referenced Datasource ID'
        referenced_datasource_class = 'Referenced Datasource Class'
        additional_groups_to_sync = 'Additional Groups To Sync'
        signal_location = 'Signal Location'
        condition_location = 'Condition Location'
        current_index_at = 'Current Index At'
        previous_index_at = 'Previous Index At'
        sync_result = 'Sync Result'
        indexing_schedule_supported = 'Indexing Schedule Supported'
        asset_tree_search_index_needs_updating = 'Asset Tree Search Index Needs Updating'
        sync_mode = 'Sync Mode'
        new_cleanup_tracking = 'New Cleanup Tracking'
        asset_progress = 'Asset Progress'
        asset_count = 'Asset Count'
        previous_asset_count = 'Previous Asset Count'
        signal_progress = 'Signal Progress'
        signal_count = 'Signal Count'
        previous_signal_count = 'Previous Signal Count'
        condition_progress = 'Condition Progress'
        condition_count = 'Condition Count'
        previous_condition_count = 'Previous Condition Count'
        scalar_progress = 'Scalar Progress'
        scalar_count = 'Scalar Count'
        previous_scalar_count = 'Previous Scalar Count'
        relationship_progress = 'Relationship Progress'
        relationship_count = 'Relationship Count'
        previous_relationship_count = 'Previous Relationship Count'
        user_group_progress = 'UserGroup Progress'
        user_group_count = 'UserGroups Count'
        previous_user_group_count = 'Previous UserGroups Count'
        interpolation_method = 'Interpolation Method'
        source_maximum_interpolation = 'Source Maximum Interpolation'
        override_maximum_interpolation = 'Override Maximum Interpolation'
        maximum_interpolation = 'Maximum Interpolation'
        key_uom = 'Key Unit Of Measure'
        value_uom = 'Value Unit Of Measure'
        source_key_uom = 'Source Key Unit Of Measure'
        source_value_uom = 'Source Value Unit Of Measure'
        units_migrated = 'Units Migrated'
        maximum_duration = 'Maximum Duration'
        cache_enabled = 'Cache Enabled'
        uncertainty_override = 'Uncertainty Override'
        metadata_properties = 'Metadata Properties'
        last_key_written = 'Last Key Written'
        request_interval_clamp = 'Request Interval Clamp'
        cache_id = 'Cache ID'
        debounce_duration = 'Debounce Duration'
        tombstoned_properties = 'Tombstoned Properties'
        replace_capsule_properties = 'Replace Capsule Properties'
        uom = 'Unit Of Measure'
        formula = 'Formula'
        swap_source_id = 'Swap Source ID'
        swap_key = 'Swap Key'
        cache_value = 'CacheValue'
        formula_parameters = 'FormulaParameters'
        expected_provided_type = 'ExpectedProvidedType'
        version_string = 'Version String'
        creator_name = 'Creator Name'
        creator_contact_info = 'Creator Contact Info'
        title = 'Title'
        examples = 'Examples'
        search_keywords = 'Search Keywords'
        experimental_package_name = 'Experimental'
        seeq_package_name = 'Seeq'
        index_doc_name = 'index'
        aggregation_function = 'AggregationFunction'
        duration = 'Duration'
        period = 'Period'
        measured_item_maximum_duration = 'MeasuredItemMaximumDuration'
        metric_configuration_migration = 'MetricReconfigurationMigration'
        bounding_condition_maximum_duration = 'BoundingConditionMaximumDuration'
        neutral_color = 'NeutralColor'
        is_generated = 'Is Generated'
        tree_type = 'Tree Type'
        manually_added = 'Manually Added'
        created_by = 'Created By'
        asset_path = 'assetPath'
        column_type = 'columnType'
        original_mapping = 'originalMapping'
        parameter_to_column = 'parameterToColumn'
        swap_source_asset_id = 'Swap Source Asset ID'
        export_data = 'Export Data'
        is_capsule_view = 'Is Capsule View'
        is_export_capsule_table = 'Is Export Capsule Table'
        is_chain_view = 'Is Chain View'
        is_autoupdate_time_range = 'Is Autoupdate Time Range'
        grid_size = 'Grid Size'
        user_email = 'User Email'
        username = 'Username'
        first_name = 'First Name'
        last_name = 'Last Name'
        password_hash = 'Password Hash'
        password_hash_method = 'Password Hash Method'
        password_set_date = 'Password Set Date'
        last_login_at = 'Last Login At'
        session_duration = 'Session Duration'
        access_key_validity_duration = 'Access Key Validity Duration'
        enabled = 'Enabled'
        admin = 'Admin'
        workbench = 'Workbench'
        user_language = 'userLanguage'
        remote_group_editable = 'Remote Group Editable'
        remove_permissions = 'Remove Permissions'
        data = 'Data'
        created_at = 'Created At'
        updated_at = 'Updated At'
        order = 'Order'
        removed_worksheet_workstep_relationships = 'Removed Worksheet Workstep Relationships'
        document = 'Document'
        plain_text_document = 'Plain Text Document'
        discoverable = 'Discoverable'
        published_at = 'Published At'
        last_viewed_at = 'Last Viewed At'
        source_security_string = 'Source Security String'
        security_string = 'Security String'
        permissions_from_datasource = 'Permissions From Datasource'
        permission_inheritance_disabled = 'Permission Inheritance Disabled'
        asset_group_member = 'Asset Group Member'
        first_admin_created = 'First Admin Created'
        u_i_config = 'UIConfig'
        dirty_cache = 'Dirty Cache'
        unmodifiable = 'Unmodifiable'
        height = 'Height'
        width = 'Width'
        scale = 'Scale'
        timezone = 'Timezone'
        selector = 'Selector'
        last_screenshot_duration = 'Last Screenshot Duration'
        average_screenshot_duration = 'Average Screenshot Duration'
        screenshot_warning = 'Screenshot Warning'
        summary_type = 'Summary Type'
        summary_value = 'Summary Value'
        current_filename = 'Current Filename'
        asset_path_depth = 'Asset Path Depth'
        resource_size = 'Resource Size'
        project_type = 'Project Type'
        cron_schedule = 'Cron Schedule'
        background = 'Background'
        condition_formula_now = 'Condition Formula Now'
        notebook_path = 'Notebook Path'
        previous_run_time = 'Previous Run Time'
        cron_schedules_to_indices = 'Cron Schedules To Indices'
        label = 'Label'
        total_run_time = 'Total Run Time'
        average_run_time = 'Average Run Time'
        is_fixed_with = 'Is Fixed Width'
        is_ck_enabled = 'Is CK Enabled'
        froala_backup = 'Froala Backup'
        original_has_workstep_guid = 'Original HasWorkstep GUID'
        icon_class = 'Icon Class'
        target_url = 'Target URL'
        link_type = 'Link Type'
        window_details = 'Window Details'
        reuse_window = 'Reuse Window'
        sort_key = 'Sort Key'
        add_on_identifier = 'Add-on Identifier'
        add_on_components = 'Add-on Components'
        html_template = 'HTML Template'
        plain_text_template = 'Plain Text Template'
        category = 'Category'
        identifier = 'Identifier'
        location = 'Location'
        entry_point = 'Entry Point'
        icon = 'Icon'
        last_update_time = 'Last Update Time'
        host = 'Host'
        options = 'Options'
        version = 'Version'
        json = 'Json'
        propagation_pending = 'Propagation Pending'
        datasource_managed = 'Datasource Managed'
        connection_id = 'ConnectionId'
        indexing_connection = 'Indexing Connection'
        pull_connection = 'Pull Connection'
        max_concurrent_requests = 'Max Concurrent Requests'
        max_results_per_requests = 'Max Results Per Requests'
        transforms = 'Transforms'
        indexing_frequency = 'Indexing Frequency'
        next_scheduled_index_at = 'Next Scheduled Index At'
        disabled_at = 'Disabled At'
        disabled_by = 'Disabled By'
        indexing_on_startup_and_config_change = 'Indexing On Startup And Config Change'
    



